@@ -27,7 +27,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: trace.c 1691 2011-02-21 21:01:08Z salcock $
+ * $Id: trace.c 1752 2012-03-05 21:18:59Z salcock $
  *
  */
 
@@ -103,6 +103,8 @@
 
 static struct libtrace_format_t *formats_list = NULL;
 
+int libtrace_halt = 0;
+
 /* strncpy is not assured to copy the final \0, so we
  * will use our own one that does
  */
@@ -498,7 +500,6 @@ DLLEXPORT int trace_config(libtrace_t *libtrace,
 		void *value)
 {
 	int ret;
-	libtrace_err_t err;
 
 	if (trace_is_err(libtrace)) {
 		return -1;
@@ -519,7 +520,7 @@ DLLEXPORT int trace_config(libtrace_t *libtrace,
 		case TRACE_OPTION_SNAPLEN:
 			/* Clear the error if there was one */
 			if (trace_is_err(libtrace)) {
-				err = trace_get_err(libtrace);
+				trace_get_err(libtrace);
 			}
 			if (*(int*)value<0 
 				|| *(int*)value>LIBTRACE_PACKET_BUFSIZE) {
@@ -531,7 +532,7 @@ DLLEXPORT int trace_config(libtrace_t *libtrace,
 		case TRACE_OPTION_FILTER:
 			/* Clear the error if there was one */
 			if (trace_is_err(libtrace)) {
-				err = trace_get_err(libtrace);
+				trace_get_err(libtrace);
 			}
 			libtrace->filter=(libtrace_filter_t *)value;
 			return 0;
@@ -1038,10 +1039,16 @@ DLLEXPORT size_t trace_get_capture_length(const libtrace_packet_t *packet)
  * not be the same as the Capture Len.
  */ 
 DLLEXPORT size_t trace_get_wire_length(const libtrace_packet_t *packet){
-	if (packet->trace->format->get_wire_length) {
-		return packet->trace->format->get_wire_length(packet);
+	
+	if (packet->wire_length == -1) {
+		if (!packet->trace->format->get_wire_length) 
+			return ~0U;
+		((libtrace_packet_t *)packet)->wire_length = 
+			packet->trace->format->get_wire_length(packet);
 	}
-	return ~0U;
+
+	assert(packet->wire_length < LIBTRACE_PACKET_BUFSIZE);
+	return packet->wire_length;
 
 }
 
@@ -1065,10 +1072,15 @@ size_t trace_get_framing_length(const libtrace_packet_t *packet) {
  * @returns libtrace_linktype_t
  */
 DLLEXPORT libtrace_linktype_t trace_get_link_type(const libtrace_packet_t *packet ) {
-	if (packet->trace->format->get_link_type) {
-		return packet->trace->format->get_link_type(packet);
+
+	if (packet->link_type == 0) {
+		if (!packet->trace->format->get_link_type)
+			return (libtrace_linktype_t)-1;
+		((libtrace_packet_t *)packet)->link_type =
+			packet->trace->format->get_link_type(packet);
 	}
-	return (libtrace_linktype_t)-1;
+
+	return packet->link_type;
 }
 
 /* process a libtrace event
@@ -1183,17 +1195,16 @@ DLLEXPORT void trace_destroy_filter(libtrace_filter_t *filter)
  * @returns -1 on error, 0 on success
  */
 static int trace_bpf_compile(libtrace_filter_t *filter,
-		const libtrace_packet_t *packet	) {
+		const libtrace_packet_t *packet,
+		void *linkptr, 
+		libtrace_linktype_t linktype	) {
 #ifdef HAVE_BPF_FILTER
-	void *linkptr = 0;
-	libtrace_linktype_t linktype;
 	assert(filter);
 
 	/* If this isn't a real packet, then fail */
-	linkptr = trace_get_packet_buffer(packet,&linktype,NULL);
 	if (!linkptr) {
 		trace_set_err(packet->trace,
-				TRACE_ERR_BAD_PACKET,"Packet has no payload");
+				TRACE_ERR_BAD_FILTER,"Packet has no payload");
 		return -1;
 	}
 	
@@ -1201,12 +1212,12 @@ static int trace_bpf_compile(libtrace_filter_t *filter,
 		pcap_t *pcap = NULL;
 		if (linktype==(libtrace_linktype_t)-1) {
 			trace_set_err(packet->trace,
-					TRACE_ERR_BAD_PACKET,
+					TRACE_ERR_BAD_FILTER,
 					"Packet has an unknown linktype");
 			return -1;
 		}
 		if (libtrace_to_pcap_dlt(linktype) == ~1U) {
-			trace_set_err(packet->trace,TRACE_ERR_BAD_PACKET,
+			trace_set_err(packet->trace,TRACE_ERR_BAD_FILTER,
 					"Unknown pcap equivalent linktype");
 			return -1;
 		}
@@ -1217,7 +1228,7 @@ static int trace_bpf_compile(libtrace_filter_t *filter,
 		assert(pcap);
 		if (pcap_compile( pcap, &filter->filter, filter->filterstring, 
 					1, 0)) {
-			trace_set_err(packet->trace,TRACE_ERR_BAD_PACKET,
+			trace_set_err(packet->trace,TRACE_ERR_BAD_FILTER,
 					"Unable to compile the filter \"%s\": %s", 
 					filter->filterstring,
 					pcap_geterr(pcap));
@@ -1243,12 +1254,20 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 	uint32_t clen = 0;
 	bool free_packet_needed = false;
 	int ret;
+	libtrace_linktype_t linktype;
 	libtrace_packet_t *packet_copy = (libtrace_packet_t*)packet;
 
 	assert(filter);
 	assert(packet);
 
-	if (libtrace_to_pcap_dlt(trace_get_link_type(packet))==~0U) {
+	/* Match all non-data packets as we probably want them to pass
+	 * through to the caller */
+	linktype = trace_get_link_type(packet);
+
+	if (linktype == TRACE_TYPE_NONDATA)
+		return 1;	
+
+	if (libtrace_to_pcap_dlt(linktype)==~0U) {
 		
 		/* If we cannot get a suitable DLT for the packet, it may
 		 * be because the packet is encapsulated in a link type that
@@ -1261,8 +1280,7 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 		packet_copy=trace_copy_packet(packet);
 		free_packet_needed=true;
 
-		while (libtrace_to_pcap_dlt(trace_get_link_type(packet_copy))==
-				~0U) {
+		while (libtrace_to_pcap_dlt(linktype) == ~0U) {
 			if (!demote_packet(packet_copy)) {
 				trace_set_err(packet->trace, 
 						TRACE_ERR_NO_CONVERSION,
@@ -1272,7 +1290,9 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 				}
 				return -1;
 			}
+			linktype = trace_get_link_type(packet_copy);
 		}
+
 	}
 	
 	linkptr = trace_get_packet_buffer(packet_copy,NULL,&clen);
@@ -1286,7 +1306,7 @@ DLLEXPORT int trace_apply_filter(libtrace_filter_t *filter,
 	/* We need to compile the filter now, because before we didn't know 
 	 * what the link type was
 	 */
-	if (trace_bpf_compile(filter,packet_copy)==-1) {
+	if (trace_bpf_compile(filter,packet_copy,linkptr,linktype)==-1) {
 		if (free_packet_needed) {
 			trace_destroy_packet(packet_copy);
 		}
@@ -1481,7 +1501,8 @@ DLLEXPORT size_t trace_set_capture_length(libtrace_packet_t *packet, size_t size
 	assert(packet);
 
 	if (packet->trace->format->set_capture_length) {
-		return packet->trace->format->set_capture_length(packet,size);
+		packet->capture_length = packet->trace->format->set_capture_length(packet,size);
+		return packet->capture_length;
 	}
 
 	return ~0U;
@@ -1767,6 +1788,8 @@ void trace_construct_packet(libtrace_packet_t *packet,
 	memcpy(packet->header,&hdr,sizeof(hdr));
 	memcpy(packet->payload,data,(size_t)len);
 	packet->type=pcap_linktype_to_rt(libtrace_to_pcap_linktype(linktype));
+
+	trace_clear_cache(packet);
 }
 
 
@@ -1814,15 +1837,25 @@ uint64_t trace_get_accepted_packets(libtrace_t *trace)
 
 void trace_clear_cache(libtrace_packet_t *packet) {
 
+	packet->l2_header = NULL;
 	packet->l3_header = NULL;
 	packet->l4_header = NULL;
+	packet->link_type = 0;
 	packet->l3_ethertype = 0;
 	packet->transport_proto = 0;
 	packet->capture_length = -1;
+	packet->wire_length = -1;
 	packet->payload_length = -1;
+	packet->l2_remaining = 0;
+	packet->l3_remaining = 0;
+	packet->l4_remaining = 0;
 
 }
 
+void trace_interrupt(void) {
+	libtrace_halt = 1;
+}
+
 void register_format(struct libtrace_format_t *f) {
 	assert(f->next==NULL); /* Can't register a format twice */
 	f->next=formats_list;
